package ctbrec.recorder.download;

import java.time.Instant;

import ctbrec.Settings;
import ctbrec.recorder.download.hls.CombinedSplittingStrategy;
import ctbrec.recorder.download.hls.NoopSplittingStrategy;
import ctbrec.recorder.download.hls.SizeSplittingStrategy;
import ctbrec.recorder.download.hls.TimeSplittingStrategy;

public abstract class AbstractDownload implements Download {

    protected Instant startTime;
    protected transient SplittingStrategy splittingStrategy;

    @Override
    public Instant getStartTime() {
        return startTime;
    }

    protected SplittingStrategy initSplittingStrategy(Settings settings) {
        SplittingStrategy strategy;
        switch (settings.splitStrategy) {
        case TIME:
            strategy = new TimeSplittingStrategy();
            break;
        case SIZE:
            strategy = new SizeSplittingStrategy();
            break;
        case TIME_OR_SIZE:
            SplittingStrategy timeSplittingStrategy = new TimeSplittingStrategy();
            SplittingStrategy sizeSplittingStrategy = new SizeSplittingStrategy();
            strategy = new CombinedSplittingStrategy(timeSplittingStrategy, sizeSplittingStrategy);
            break;
        case DONT:
        default:
            strategy = new NoopSplittingStrategy();
            break;
        }
        strategy.init(settings);
        return strategy;
    }
}
